package com.example.android.clouddb;


import android.os.Parcel;
import android.os.Parcelable;
import com.google.firebase.database.DataSnapshot;
import com.google.firebase.database.DatabaseError;
import com.google.firebase.database.DatabaseReference;
import com.google.firebase.database.FirebaseDatabase;
import com.google.firebase.database.ValueEventListener;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Map;
import java.util.Observable;
import java.util.Random;
import java.util.Set;


public class DBHandler extends Observable implements Parcelable {
    // Database child category names
    private final String CATGETORY_PREFERENCES = "Preferences";
    private final String CATEGORY_RECIPES = "Recipes";
    private final String userDataKey = "UserData";


    private HashMap<String, Recipe> recipes;
    private ArrayList updatedKeys;
    private FirebaseDatabase db;
    private DatabaseReference dbReference;
    private ValueEventListener listener;
    private Random randomGenerator;
    private MainActivity mainActivity = null;
    private Map userPreferences;

    public DBHandler() {    init(null, null);    }

    public DBHandler(MainActivity activity, String uuid) {init(activity, uuid);
    }

    public void init(MainActivity activity, String uuid) {

        this.mainActivity = activity;

        // Initialize connection to database
        db = FirebaseDatabase.getInstance();
        db.goOnline();  // Resumes cloud syncing

        // Set up global vars
        recipes = new HashMap<String, Recipe>();  // eventListeners will fill in this data on change notifications
        updatedKeys = new ArrayList();
        randomGenerator = new Random();
        userPreferences = new HashMap();

        dbReference = db.getReference().child(userDataKey);  // set default db reference to UserData folder
        if (uuid != null) {
            dbReference = dbReference.child(uuid); // If user id provided, descend into user's folder
        }

        // Disable auto-clear cache for offline mode
        dbReference.keepSynced(true);

        // Start listeners
        setRecipeDBReferenceListener(dbReference.child(CATEGORY_RECIPES));
        setPreferencesReferenceListener(dbReference.child(CATGETORY_PREFERENCES));

    }

    public void setRecipeDBReferenceListener(DatabaseReference reference) {
        listener = new ValueEventListener() {
            @Override
            public void onDataChange(DataSnapshot dataSnapshot) {
                recipes.clear();
                for (DataSnapshot data : dataSnapshot.getChildren()) {
                    Recipe recipe = data.getValue(Recipe.class);
                    recipes.put(recipe.getRecipeName(), recipe);
                    updatedKeys.add(recipe.getRecipeName());
                }
                setChanged();
                notifyObservers();
            }


            @Override
            public void onCancelled(DatabaseError databaseError) {
                System.out.println("RPW - " + databaseError.toString());
            }
        };
        
        reference.addValueEventListener(listener);
    }

    public void setPreferencesReferenceListener(DatabaseReference reference) {
        listener = new ValueEventListener() {
            @Override
            public void onDataChange(DataSnapshot dataSnapshot) {
                userPreferences.clear();
                for (DataSnapshot data : dataSnapshot.getChildren()) {
                    userPreferences.put(data.getKey(), data.getValue());
                }
                setChanged();
                notifyObservers();
            }

            @Override
            public void onCancelled(DatabaseError databaseError) {
                System.out.println("RPW - " + databaseError.toString());
            }
        };
        reference.addValueEventListener(listener);
    }

    public void enableSync() {
        db.goOnline();  // Resumes cloud syncing
    }

    public void updateMealPlan(int mealID, ArrayList<String> mealRecipes, Boolean tagValue) {
        for (String recipeName : mealRecipes) {
            Recipe recipe = recipes.get(recipeName);
            recipe.tagMeal(mealID, tagValue);
            replaceRecipe(recipe, recipe);
        }
    }

    public void replaceRecipe(Recipe recipeOld, Recipe recipeNew) {
        recipeNew.setMealTags(recipeOld.getMealTags());
        dbReference.child(CATEGORY_RECIPES).child(recipeOld.getRecipeName()).removeValue();
        dbReference.child(CATEGORY_RECIPES).child(recipeNew.getRecipeName()).setValue(recipeNew);
    }

    public Recipe getRecipeByID(String id) {
        return (Recipe) recipes.get(id);
    }

    public HashMap getRecipes() {
        return recipes;
    }

    // Gets "random" recipe from databsae
    // Ignores recipes that do not match user's criteria
    // MUST have vaild this.MainActivity
    public Recipe getRandomRecipe() {
        if (mainActivity != null) {
            ArrayList<String> possibleRecipes = new ArrayList<String>();
            possibleRecipes.addAll(updatedKeys);
            Set<String> userTags = mainActivity.getUserTags();
            while (possibleRecipes.size() > 0) {
                int random = randomGenerator.nextInt(possibleRecipes.size());
                Recipe recipeCandidate = recipes.get(possibleRecipes.get(random));
                System.out.println("RPW - RecipeCandidate: " + recipeCandidate.toString());
                if (recipeCandidate != null) {
                    if ((userTags == null) || (recipeCandidate.satisfiesTags(userTags))) {
                        return recipeCandidate;
                    } else {
                        possibleRecipes.remove(recipeCandidate.getRecipeName());
                    }
                }
            }

        }
        return null;
    }

    public HashMap<String, Recipe> getRecipesWithMealTag(int mealTag) {
        HashMap<String, Recipe> subset = new HashMap<>();
        for (String key : recipes.keySet()) {
            if (recipes.get(key).isTaggedForMeal(mealTag)) {
                subset.put(key, recipes.get(key));
            }
        }
        return subset;
    }

    public Map getUserPreferences() {
        return userPreferences;
    }

    public void setUserPreferences(Map userPreferences) {
        dbReference.child(CATGETORY_PREFERENCES).removeValue();
        dbReference.child(CATGETORY_PREFERENCES).setValue(userPreferences);

    }

    public void close() {
        setChanged();
        notifyObservers();
        dbReference.removeEventListener(listener);
        //db.goOffline();  // Shuts down connection to cloud database
    }

    @Override
    public int describeContents() {
        return 0;
    }

    @Override
    public void writeToParcel(Parcel parcel, int i) { }

    public static final Parcelable.Creator<DBHandler> CREATOR
            = new Parcelable.Creator<DBHandler>() {
        public DBHandler createFromParcel(Parcel in) {
            return new DBHandler();
        }

        public DBHandler[] newArray(int size) {
            return new DBHandler[size];
        }
    };

}


