package com.example.android.clouddb;

import android.content.BroadcastReceiver;
import android.content.Context;
import android.content.Intent;
import android.content.IntentFilter;
import android.content.SharedPreferences;
import android.content.res.Configuration;
import android.preference.PreferenceManager;
import android.support.annotation.Nullable;
import android.support.v4.app.Fragment;
import android.support.v4.app.FragmentManager;
import android.support.v4.app.FragmentTransaction;
import android.support.v4.widget.DrawerLayout;
import android.support.v7.app.ActionBarDrawerToggle;
import android.support.v7.app.AppCompatActivity;
import android.os.Bundle;
import android.view.MenuItem;
import android.view.View;
import android.widget.AdapterView;
import android.widget.ArrayAdapter;
import android.widget.ListView;

import com.firebase.ui.auth.AuthUI;
import com.firebase.ui.auth.IdpResponse;
import com.google.firebase.auth.FirebaseAuth;
import com.google.firebase.database.FirebaseDatabase;

import java.util.Arrays;
import java.util.HashSet;
import java.util.List;
import java.util.Map;
import java.util.Observable;
import java.util.Observer;
import java.util.Set;

public class MainActivity extends AppCompatActivity implements Observer{

    private DBHandler database;

    // For nav bar
    private ListView aNavList;
    private ActionBarDrawerToggle aDrawerToggle;
    private DrawerLayout aDrawerLayout;
    private ArrayAdapter<String> mAdapter;



    private FragmentManager fragmentManager;
    private final String CONNECTION_FILTER = "android.net.conn.CONNECTIVITY_CHANGE";
    private String anActivityTitle = "Recipe Book";
    private DBDisplay homeDisplay;
    private BroadcastReceiver receiver;


    // FIREBASE keys
    private static final int RC_SIGN_IN = 123;
    List<AuthUI.IdpConfig> providers = Arrays.asList(new AuthUI.IdpConfig.Builder(AuthUI.EMAIL_PROVIDER).build());


    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        triggerLogIn();
        setContentView(R.layout.activity_main);
        aNavList = (ListView) findViewById(R.id.navList);
        aDrawerLayout = (DrawerLayout) findViewById(R.id.drawer_layout);
        addDrawerItems();
        setupDrawer();

        registerNetworkReceiver();
    }


    public void registerNetworkReceiver() {

        IntentFilter filter = new IntentFilter();
        filter.addAction(CONNECTION_FILTER);

        receiver = new BroadcastReceiver() {
            @Override
            public void onReceive(Context context, Intent intent) {
                if (database != null) {
                    database.enableSync();
                }
            }
        };
        registerReceiver(receiver, filter);

    }

    public void triggerLogIn() {
        if (FirebaseAuth.getInstance().getCurrentUser() != null) {
            // restoring session from saved login
            FirebaseDatabase.getInstance().setPersistenceEnabled(true);
            connectDBWithUser(FirebaseAuth.getInstance().getUid());
            loadUserSettings();
            showDBContents();
        }
        else {
            // Create and launch sign-in intent
            startActivityForResult(
                    AuthUI.getInstance()
                            .createSignInIntentBuilder()
                            .setAvailableProviders(providers)
                            .build(),
                    RC_SIGN_IN);
        }

    }

    public void connectDBWithUser(String uuid) {
        database = new DBHandler(this, uuid);
    }

    public void loadUserSettings() {
        database.addObserver(this);
    }

    @Override
    protected void onActivityResult(int requestCode, int resultCode, Intent data) {
        super.onActivityResult(requestCode, resultCode, data);
        if (requestCode == RC_SIGN_IN) {
            if (resultCode == RESULT_OK) {
                connectDBWithUser(FirebaseAuth.getInstance().getUid());
                loadUserSettings();  //
                showDBContents();

            }
            else {
                // Sign in failed, close app
                System.out.println("RPW - User failed to log in");
                this.finishAffinity();
            }
        }
    }

    public void showDBContents() {
        homeDisplay = DBDisplay.newInstance(database);
        addFragmentToStack(homeDisplay);
        homeDisplay.setInitialData(database.getRecipes());

    }

    public DBHandler getDatabase() {
        return this.database;
    }

    public void addDrawerItems() {
        String [] menuItems = getResources().getStringArray(R.array.nav_items);
        mAdapter = new ArrayAdapter<>(this, android.R.layout.simple_list_item_1, menuItems);
        aNavList.setAdapter(mAdapter);

        aNavList.setOnItemClickListener(new AdapterView.OnItemClickListener() {
            @Override
            public void onItemClick(AdapterView<?> parent, View view, int position, long id) {
                Fragment newView = null;

                switch (position) {
                    case 0:  // Home button
                        newView = homeDisplay;
                        break;
                    case 1:  // Add recipe button
                        newView = new AddForm();
                        break;
                    case 2: // Search button
                        newView = new QueryForm();
                        break;
                    case 3: // Create Meal Plan button
                        newView = new MealPlanner3();
                        break;
                    case 4: // Create Grocery List Generator button
                        newView = new GroceryDisplay();
                        break;
                    case 5: // Settings Page
                        newView = new SettingsFragment();
                        break;
                    case 6: // Log off button
                        logOffUser();
                        break;
                }
                if (newView != null) {
                    replaceFragment(newView, R.id.fragment_container);
                }
            }
        });

    }

    public void addFragmentToStack(Fragment fragment) {
        fragmentManager = getSupportFragmentManager();
        FragmentTransaction transaction = fragmentManager.beginTransaction();
        transaction
                .add(R.id.fragment_container, fragment)
                .addToBackStack(null)
                .commit();
    }


    public void replaceFragment(Fragment fragment, int fragmentID) {
        fragmentManager = getSupportFragmentManager();
        FragmentTransaction transaction = fragmentManager.beginTransaction();
        transaction
                .replace(fragmentID, fragment)
                .addToBackStack(null)
                .commit();
    }


    // Gets local shared preferences
    public Set<String> getUserTags() {
        SharedPreferences prefs = PreferenceManager.getDefaultSharedPreferences(this);
        Set<String> userTags = new HashSet<String>();
        for (String key : getResources().getStringArray(R.array.user_tags)) {
            if (prefs.getBoolean(key, false)) {
                userTags.add(key);
            }
        }
        return userTags;
    }


    public void setupDrawer() {
        aDrawerToggle = new ActionBarDrawerToggle(this, aDrawerLayout, R.string.drawer_open, R.string.drawer_close) {
            public void onDrawerOpened(View drawerView) {
                super.onDrawerOpened(drawerView);
                getSupportActionBar().setTitle("Navigation");
                invalidateOptionsMenu();
            }
            public void onDrawerClosed(View view) {
                super.onDrawerClosed(view);
                getSupportActionBar().setTitle(anActivityTitle);
                invalidateOptionsMenu();
            }

        };
        aDrawerToggle.setDrawerIndicatorEnabled(true);
        aDrawerLayout.setDrawerListener(aDrawerToggle);

    }

    // Logs off and closes app
    public void logOffUser(){
        // disconnect database
        database.close();
        FirebaseAuth.getInstance().signOut();
        this.finishAffinity();
    }

    @Override
    public boolean onOptionsItemSelected(MenuItem item) {
        if (aDrawerToggle.onOptionsItemSelected(item)) {
            return true;
        }
        return super.onOptionsItemSelected(item);
    }

    @Override
    protected void onPostCreate(@Nullable Bundle savedInstanceState) {
        super.onPostCreate(savedInstanceState);
        aDrawerToggle.syncState();
    }


    @Override
    public void onConfigurationChanged(Configuration newConfig) {
        super.onConfigurationChanged(newConfig);
        aDrawerToggle.onConfigurationChanged(newConfig);
    }


    // If shared preferences on cloud are different
    // than local, overwrite local with cloud
    @Override
    public void update(Observable observable, Object o) {
        Map userPreferences = database.getUserPreferences();
        SharedPreferences preferences = PreferenceManager.getDefaultSharedPreferences(this);
        Map preferenceMap = preferences.getAll();

        // ONLY call updatePreferences if the preferences have changed.
        // Risk infinite update loop if call updatePreferences with
        // unchanged settings
        if (!preferenceMap.keySet().equals(userPreferences.keySet())) {
            updatePreferences(userPreferences);
        }
        for (Object key : preferenceMap.keySet()) {
            if (!userPreferences.containsKey(key) || (userPreferences.get(key) != preferenceMap.get(key))) {
                updatePreferences(userPreferences);
            }
        }
    }

    public void updatePreferences(Map userPreferences) {
        SharedPreferences preferences = PreferenceManager.getDefaultSharedPreferences(this);
        preferences.edit().clear().commit();
        for (Object key : userPreferences.keySet()) {
            System.out.println("RPW - updating key: " + key.toString());
            preferences.edit().putBoolean(key.toString(), (Boolean) userPreferences.get(key)).commit();
        }
    }


    @Override
    protected void onStop() {
        super.onStop();
        if (database != null) {
            database.close();
        }
    }

    @Override
    protected void onDestroy() {
        super.onDestroy();
        if (database != null) {
            database.close();
        }
    }
}


